﻿using Infragistics.Win.UltraWinDock;
using System;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {

        private string _layoutXml = string.Empty;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SaveLayout();
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            SaveLayout();
        }

        private void ultraButton2_Click(object sender, EventArgs e)
        {
            ApplyLayout();
        }
        
        void ApplyLayout()
        {
            using (var stream = new System.IO.MemoryStream(Encoding.UTF8.GetBytes(_layoutXml)))
            {
                ultraDockManager1.LoadFromXML("MyFile.xml");
            }
        }

        void SaveLayout()
        {
            using (var stream = new System.IO.MemoryStream())
            {
                ultraDockManager1.SaveAsXML("MyFile.xml");
                _layoutXml = Encoding.UTF8.GetString(stream.ToArray());
            }
        }

        private const int MinimumUndockedWidth = 25;
        private const int MinimumUndockedHeight = 25;

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            Rectangle undockedArea = CalculateUndockedArea(this);

            if (undockedArea.Width < MinimumUndockedWidth)
                ReduceDockArea(this.ultraDockManager1, MinimumUndockedWidth - undockedArea.Width, Orientation.Horizontal);

            if (undockedArea.Height < MinimumUndockedHeight)
                ReduceDockArea(this.ultraDockManager1, MinimumUndockedHeight - undockedArea.Height, Orientation.Vertical);
        }

        private IAsyncResult asyncCalcMinSize;

        protected override void OnLayout(LayoutEventArgs e)
        {
            base.OnLayout(e);

            if (this.asyncCalcMinSize == null || this.asyncCalcMinSize.IsCompleted)
            {
                if (!this.IsHandleCreated)
                    this.CalculateMinimumSize();
                else
                    this.asyncCalcMinSize = this.BeginInvoke(new MethodInvoker(this.CalculateMinimumSize));
            }
        }

        private void CalculateMinimumSize()
        {
            if (!this.Disposing && !this.IsDisposed)
            {
                Size minSize = CalculateMinimumSize(this.ultraDockManager1);

                minSize.Width += MinimumUndockedWidth;
                minSize.Height += MinimumUndockedHeight;

                this.MinimumSize = minSize;
            }
        }

        #region CalculateMinimumSize
        /// <summary>
        /// Calculates the minimum size for the control hosting the ultradockmanger based on the controls which are docked and the minimum size of the dock areas
        /// </summary>
        /// <param name="dockManager">Dock manager whose container is to be evaluated</param>
        /// <returns>A minimum size based on the currently visible controls in the dockmanager' host control and the current extent of the docked dockarea panes.</returns>
        private static Size CalculateMinimumSize(UltraDockManager dockManager)
        {
            Size dockedSize = Size.Empty;

            if (dockManager.HostControl != null)
            {
                // the non-client area of the container must
                // also be taken into account
                dockedSize = dockManager.HostControl.Size - dockManager.HostControl.ClientSize;

                foreach (Control child in dockManager.HostControl.Controls)
                {
                    if (!child.Visible)
                        continue;

                    // if this is a dockarea pane then add in the minimum size of the pane
                    if (child is WindowDockingArea)
                    {
                        DockAreaPane dockArea = ((WindowDockingArea)child).Pane;

                        switch (dockArea.DockedLocation)
                        {
                            case DockedLocation.DockedLeft:
                            case DockedLocation.DockedRight:
                                dockedSize.Width += dockArea.MinimumSizeResolved.Width;
                                break;
                            case DockedLocation.DockedTop:
                            case DockedLocation.DockedBottom:
                                dockedSize.Height += dockArea.MinimumSizeResolved.Height;
                                break;
                        }
                    }
                    else
                    {
                        // for all other controls just use their current size
                        switch (child.Dock)
                        {
                            case DockStyle.Left:
                            case DockStyle.Right:
                                dockedSize.Width += child.Width;
                                break;
                            case DockStyle.Top:
                            case DockStyle.Bottom:
                                dockedSize.Height += child.Height;
                                break;
                        }
                    }
                }
            }

            return dockedSize;
        }
        #endregion //CalculateMinimumSize

        #region ReduceDockArea
        /// <summary>
        /// Adjusts the size of the specified dockareas by the specified amount
        /// </summary>
        /// <param name="dockManager">Dockmanager whose dock areas are to be adjusted</param>
        /// <param name="adjustment">Amount to decrease the extent of the dockareas</param>
        /// <param name="orientation">Location of the dock areas that should be reduced - Horizontal for the left/right dockareas and Vertical for the top/bottom dockareas</param>
        private static void ReduceDockArea(UltraDockManager dockManager, int adjustment, Orientation orientation)
        {
            for (int i = dockManager.DockAreas.Count - 1; i >= 0; i--)
            {
                // leave if we've adjusted enough
                if (adjustment <= 0)
                    return;

                DockAreaPane dockArea = dockManager.DockAreas[i];

                switch (dockArea.DockedLocation)
                {
                    case DockedLocation.Floating:
                        // don't touch the floating the dockareas
                        break;
                    case DockedLocation.DockedLeft:
                    case DockedLocation.DockedRight:
                        if (orientation == Orientation.Horizontal)
                        {
                            int minimum = dockArea.MinimumSizeResolved.Width;
                            int extent = dockArea.Size.Width;
                            int newExtent = Math.Max(extent - adjustment, minimum);

                            // decrease the adjustment by the adjustment
                            // we are making to this dockarea
                            adjustment -= (extent - newExtent);

                            dockArea.Size = new Size(newExtent, dockArea.Size.Height);
                        }
                        break;
                    case DockedLocation.DockedTop:
                    case DockedLocation.DockedBottom:
                        if (orientation == Orientation.Vertical)
                        {
                            int minimum = dockArea.MinimumSizeResolved.Height;
                            int extent = dockArea.Size.Height;
                            int newExtent = Math.Max(extent - adjustment, minimum);

                            // decrease the adjustment by the adjustment
                            // we are making to this dockarea
                            adjustment -= (extent - newExtent);

                            dockArea.Size = new Size(dockArea.Size.Width, newExtent);
                        }
                        break;
                }
            }
        }
        #endregion //ReduceDockArea

        #region CalculateUndockedArea
        /// <summary>
        /// Calculates the amount of space within a specified control that is not occupied by docked controls.
        /// </summary>
        /// <param name="container">Control whose content area should be calculated</param>
        /// <returns>A rectangle indicating the area for the filled control. Note, the width and/or height may be a negative value.</returns>
        private static Rectangle CalculateUndockedArea(Control container)
        {
            // start with the area where controls can be docked
            Rectangle rect = container.DisplayRectangle;

            foreach (Control child in container.Controls)
            {
                if (!child.Visible)
                    continue;

                switch (child.Dock)
                {
                    case DockStyle.Top:
                        rect.Y += child.Height;
                        rect.Height -= child.Height;
                        break;
                    case DockStyle.Bottom:
                        rect.Height -= child.Height;
                        break;
                    case DockStyle.Left:
                        rect.X += child.Width;
                        rect.Width -= child.Width;
                        break;
                    case DockStyle.Right:
                        rect.Width -= child.Width;
                        break;
                }
            }

            return rect;
        }
        #endregion //CalculateUndockedArea
    }
}
